import { classOf, nameOf } from "@tsed/core";
import { constantCase } from "change-case";
import statuses from "statuses";
export class Exception extends Error {
    static validate(status, family) {
        if (status < 100 || status >= 600) {
            throw new Error("Status must be between 100 and 599");
        }
        if (status.toString()[0] !== family.charAt(0)) {
            throw new Error(`Status ${status} does not belong to the family ${family}`);
        }
        return status;
    }
    constructor(status = 500, message = "", origin) {
        super(message);
        /**
         * Exception base name
         * @type {string}
         */
        this.name = "HTTP_EXCEPTION";
        /**
         * Exception type
         * @type {string}
         */
        this.type = "HTTP_EXCEPTION";
        /**
         * Headers must be sent to the response
         */
        this.headers = {};
        this.status = status;
        this.message = message;
        this.name = this.#getStatusConstant();
        this.setOrigin(origin);
        Error.captureStackTrace(this);
    }
    setHeaders(headers) {
        this.headers = {
            ...this.headers,
            ...headers
        };
        return this;
    }
    setHeader(key, value) {
        this.headers[key] = value;
        return this;
    }
    setOrigin(origin) {
        if (origin) {
            if (origin instanceof Error) {
                this.origin = origin;
                this.message = `${this.message}, innerException: ${this.origin.message}`.trim();
            }
            else if (typeof origin === "string") {
                this.origin = new Error(origin);
                this.message = `${this.message}, innerException: ${this.origin.message}`.trim();
            }
            else {
                this.body = origin;
            }
        }
    }
    toString() {
        return `${this.name}(${this.status}): ${this.message} `.trim();
    }
    #getStatusConstant() {
        try {
            return this.status === 200 ? "SUCCESS" : constantCase(String(statuses(this.status)));
        }
        catch (er) {
            return constantCase(nameOf(classOf(this)));
        }
    }
}
export class HTTPException extends Exception {
}
